body {
margin: 0;
background-color: #fff;
color: #00008f;
font-family: arial,helvetica,sans-serif;
font-size: 12px;
}

h1 {
font-size: 14px;
color: #00378a;
}

h2,h3,h4,h5,h6 {
font-size: 13px;
color: #00378a;
}

td,p,b,th {
font-size: 12px;
}

img {
border: none;
}

a:link {
text-decoration: none;
color: #00008f;
}

a:visited {
text-decoration: none;
color: #00008f;
}

td.listfooter {
background: #e5e5e5;
border-top: 1px solid silver;
padding: 3px;
height: 16px;
}

td.scrollhead {
vertical-align: top;
padding: 0;
}

td.scrollbody {
vertical-align: top;
padding: 0;
}

div.scrollbody {
background-color: #fff;
overflow: auto;
text-align: left;
}
html.rtl div.scrollbody {
text-align: right;
}

/* Title bar */
a.maintitlebar,div.maintitlebar {
font-family: arial,helvetica,sans-serif;
text-decoration: none;
font-size: 12px;
text-align: center;
vertical-align: middle;
cursor: pointer;
display: inline-block;
padding: 2px;
border-radius: 8px;
transition: box-shadow .5s ease, background .5s ease;
}
html.ltr a.maintitlebar,html.ltr div.maintitlebar {
padding-right: 5px;
}
html.rtl a.maintitlebar,html.rtl div.maintitlebar {
padding-left: 5px;
}

a.maintitlebar:hover {
text-decoration: none;
background-color: #d7e6b4;
cursor: pointer;
box-shadow: 0px 0px 2px black;
}

td.phonelist:hover {
background-color: #d2d2d2;
}

td.phonelist {
border-right: 1px solid #b0b0b0;
}

div.copynotice {
border-style: solid;
border-color: #aaa;
width: 100%;
border-top-width: 1px;
border-bottom-width: 0;
border-left-width: 0;
border-right-width: 0;
padding-top: 3px;
padding-bottom: 0;
text-align: right;
font-family: arial,helvetica,sans-serif;
font-size: 10px;
}
html.rtl div.copynotice {
text-align: left;
}

hr {
width: 100%;
border: none;
background-color: #aaa;
height: 1px;
}

.must {
color: red;
font-family: arial,helvetica,sans-serif;
}

img.center {
text-align: center;
vertical-align: middle;
}

input.center {
text-align: left;
vertical-align: middle;
}
html.rtl input.center {
text-align: right;
}

select.center {
text-align: left;
vertical-align: middle;
}
html.rtl select.center {
text-align: right;
}

a.center {
text-align: left;
vertical-align: middle;
}
html.rtl a.center {
text-align: right;
}

/********************** Template migration *********************/

form#mainform {
margin: 0;
}

table.framework {
height: auto;
width: 100%;
border: 0;
border-spacing: 5px;
padding: 0;
margin-top: 0;
vertical-align: top;
box-shadow: 0px 0px 2px black;
}
html.ltr table.framework {
margin-left: 0;
}
html.rtl table.framework {
margin-right: 0;
}

table.framework td#maincell {
background-color: white;
width: auto;
vertical-align: top;
padding-bottom: 35px;
}

/* cellpadding="0" */
table.framework > tbody > tr > td {
padding: 0;
}

table.maintab {
width: 100%;
height: 100%;
border: 0;
border-spacing: 0;
padding: 0;
margin: 0;
background-color: #507aaa;
vertical-align: top;
}

div.contentboxh {
background-color: #f0f0f0;
border: 1px solid #aaa;
border-bottom: none;
height: 26px;
vertical-align: middle;
width: auto;
}

div.contentboxh > p {
margin: 2px;
font-family: arial,helvetica,sans-serif;
font-size: 16px;
font-weight: bold;
vertical-align: middle;
}

div.contentboxb {
border: 1px solid #b0b0b0;
border-top-width: 0;
vertical-align: middle;
width: auto;
background: white;
padding: 5px;
}
.filter div.contentboxb {
padding: 0;
}
div.contentboxb > div {
display: inline-block;
margin: 1px;
}
.filter div.contentboxb > div {
width: 100%;
background: #eeeeee;
border-top: 1px solid #b0b0b0;
text-align: right;
margin: 0;
padding: 3px;
box-sizing: border-box;
}
.filter div.contentboxb > label img {
vertical-align: middle;
padding: 3px;
}
.filter div.contentboxb > hr {
margin-bottom: 3px;
}
div.contentboxb > img, div.contentboxb > input {
display: inline-block;
margin: 3px;
}

p.contentboxb {
padding-bottom: 0;
margin: 0;
border: 4px solid #f8f8f8;
vertical-align: middle;
}

a.alphaselect:hover {
text-decoration: none;
background-color: #a0a0a0;
color: #fff;
}

table.check {
background-color: #e1e1f1;
color: #000;
border: 1px solid #aaa;
width: 95%;
}
html.ltr table.check {
margin-left: 20px;
}
html.rtl table.check {
margin-right: 20px;
}

html.ltr td.check {
border-right: 1px solid #aaa;
}
html.rtl td.check {
border-left: 1px solid #aaa;
}

option.select {
background-repeat: no-repeat;
background-position: 0 top;
border: 0;
padding-bottom: 1px;
height: 18px;
}
html.ltr option.select {
padding-left: 20px;
}
html.rtl option.select {
padding-right: 20px;
}


td.tbhead {
border-bottom: 1px solid #b0b0b0;
}
html.ltr td.tbhead {
border-right: 1px solid #b0b0b0;
}
html.rtl td.tbhead {
border-left: 1px solid #b0b0b0;
}

td.tbrhead {
border-bottom: 1px solid #b0b0b0;
}

.item {
text-align: left;
white-space: nowrap;
font-family: "bitstream vera sans", "luxi sans", verdana, geneva, arial, helvetica, sans-serif;
font-size: 12px;
padding-top: 2px;
}
html.rtl .item {
text-align: right;
}

.item div {
position: relative;
display: block;
text-decoration: none;
}

.item :hover {
color: #fff;
background-color: #4b6983;
text-decoration: none;
}

.separator {
background-color: red;
border-top: 1px solid #c6c6bf;
border-bottom: 1px solid #fff;
margin: 2px 5px;
/* top right bottom left */
height: 0;
font-size: 1px;
line-height: 0;
}

div.autocomplete {
position: absolute;
background-color: #fff;
border: 1px solid #aaa;
margin: 0;
padding: 0;
z-index: 600;
overflow: hidden;
word-wrap: break-word;
}

div.autocomplete ul {
list-style-type: none;
margin: 0;
padding: 0;
}

div.autocomplete ul li {
list-style-type: none;
display: block;
margin: 0;
padding: 2px;
cursor: pointer;
}
html.ltr div.autocomplete ul li {
padding-left: 4px;
}
html.rtl div.autocomplete ul li {
padding-right: 4px;
}


div.autocomplete li:hover {
background-color: #f0f0f0;
}

div.autocomplete ul li.selected {
background-color: #f0f0f0;
}

#pulldown {
background: #fff;
height: 23px;
border-top: 1px #d0d0d0 solid;
border-bottom: 1px gray solid;
}
html.ltr #pulldown {
border-left: 1px #d0d0d0 solid;
border-right: 1px gray solid;
}
html.rtl #pulldown {
border-right: 1px #d0d0d0 solid;
border-left: 1px gray solid;
}

#pulldown ul {
display: block;
margin: 0;
padding: 0;
line-height: 1em;
list-style: none;
z-index: 90;
}

#pulldown ul li {
margin: 0 3px 0 0;
padding: 0;
font-size: 12px;
line-height: 1em;
list-style-type: none;
}
html.rtl #pulldown ul li {
margin: 0 0 0 3px;
}

#pulldown ul li a {
float: left;
display: block;
width: auto;
font-weight: normal;
background: transparent;
text-decoration: none;
margin: 0;
padding: 5px;
}
html.rtl #pulldown ul li a {
float: right;
}

#pulldown ul li a:hover {
text-decoration: none;
}

#pulldown ul li.sep {
color: #aaa;
padding: .8em 0 .5em;
}

/* Commented Backslash Hack hides rule from IE5-Mac \*/
#pulldown ul li a {
float: none;
}

/* End IE5-Mac hack */
#pulldown ul ul {
position: absolute;
top: 0;
left: 0;
visibility: hidden;
border-top: 1px #d0d0d0 solid;
border-left: 1px #d0d0d0 solid;
border-bottom: 1px gray solid;
border-right: 1px gray solid;
background: #fff;
}

#pulldown ul ul li {
border-bottom: 1px solid #fff;
float: none;
margin: 0;
padding: 0;
width: 200px;
}

#pulldown ul ul li a {
padding: 5px 9px 5px 5px;
}

#pulldown ul ul li a:hover {
font-weight: normal;
background-color: #418dd4;
background-image: none;
}

span.informal {
color: #444;
font-style: italic;
}

span.mark {
color: #b22;
}

#debug-handling {
border-color: #ddd;
}
.error, .notice, .success, .info {
border: 2px solid #ddd;
margin: .2em;
padding: .3em;
}
.error {
background: #fbe3e4;
color: #8a1f11;
border-color: #fbc2c4;
}
.error h1,.error h2,.error h3,.error h4,.error h5,.error h6 {
color: #8a3700;
}
.notice {
background: #fff6bf;
color: #514721;
border-color: #ffd324;
}
.notice h1,.notice h2,.notice h3,.notice h4,.notice h5,.notice h6 {
color: #412701;
}
.success {
background: #e6efc2;
color: #264409;
border-color: #c6d880;
}
.success h1,.success h2,.success h3,.success h4,.success h5,.success h6 {
color: #378a00;
}
.info {
background: #d5edf8;
color: #205791;
border-color: #92cae4;
}
.info h1,.info h2,.info h3,.info h4,.info h5,.info h6 {
color: #00378a;
}

/* Header */

html.ltr div#header-left {
float: left;
}
html.rtl div#header-left {
float: right;
}


.plugtop {
border: none;
border-radius: 10px;
box-shadow: 0px 0px 4px black inset;
padding: 10px;
margin: 0;
text-align: center;
vertical-align: middle;
background-color: #dbe6c6;
font-family: arial,helvetica,sans-serif;
font-size: 18px;
}
.plugtop img {
height: 32px;
}
html.ltr .plugtop img {
margin-right: 4px;
}
html.rtl .plugtop img {
margin-left: 4px;
}
html.ltr div#header-left .plugtop {
margin-left: 35px;
}
html.rtl div#header-left .plugtop {
margin-right: 35px;
}

div#header-left img {
vertical-align: middle;
}

html.ltr div#header-left img#fd-logo {
padding-left: 2px;
padding-right: 25px;
}
html.rtl div#header-left img#fd-logo {
padding-left: 25px;
padding-right: 2px;
}

html.ltr div#header-left .logout {
margin-left: 35px;
}
html.rtl div#header-left .logout {
margin-right: 35px;
}

div#header-right {
padding-top: 8px;
height: 38px;
color: black;
}
html.ltr div#header-right {
text-align: right;
}
html.rtl div#header-right {
text-align: left;
}
div#header-right div.version {
font-size: 20px;
}

/* Header col */
div.setup-header {
background-color: #bbcc99;
padding: 4px 5px 2px;
color: #000;
}

/* On small screens */
@media (max-width: 640px) {
  div.setup-header {
  padding: 2px;
  height: 36px;
  }
  a.maintitlebar {
  width: 36px;
  height: 36px;
  overflow: hidden;
  display: inline-block;
  }
  html.ltr a.maintitlebar {
  float: left;
  margin: 0;
  padding: 0;
  }
  html.rtl a.maintitlebar {
  float: right;
  margin: 0;
  }
  a.maintitlebar img {
  padding: 6px;
  width: 22px;
  height: 22px;
  }
  html.ltr div#header-left .logout {
  margin-left: 0;
  float: right;
  }
  html.rtl div#header-left .logout {
  margin-right: 0;
  float: left;
  }
  html.ltr div#header-left, html.rtl div#header-left {
  height: 40px;
  width: 100%;
  float: none;
  }
  html.ltr div#header-right, html.rtl div#header-right {
  padding-top: 8px;
  height: 30px;
  margin-top: -40px;
  width: 100%;
  float: none;
  }
  div#header-right a {
  background: #bbcc99;
  }
  .optional {
  display: none !important;
  }
  div#header-left .plugtop {
  display: inline-block;
  hyphens: none;
  overflow: hidden;
  white-space: nowrap;
  padding-top: 2px;
  padding-bottom: 2px;
  font-size: 1em;
  }
  html.ltr div#header-left .plugtop {
  margin: 0;
  padding-left: 6px;
  }
  html.rtl div#header-left .plugtop {
  margin: 0;
  padding-right: 6px;
  }
  div.logout-label {
  display: inline-block;
  vertical-align: middle;
  }
  table.framework {
  margin: 0;
  border: none;
  width: 100%;
  border-spacing: 0;
  }
  table.framework #maincell
  {
  padding-bottom: 55px;
  }
}
