<?php
/*
  This code is part of FusionDirectory (http://www.fusiondirectory.org/)
  Copyright (C) 2013-2018  FusionDirectory

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/*!
 * \brief User and/or POSIX group selection
 */
class userGroupSelect extends userSelect
{
  protected $objectTypes = array('user', 'group');
  protected $autoFilterAttributes = array('dn', 'cn', 'uid', 'description', 'mail');

  function __construct()
  {
    if (class_available('mixedGroup')) {
      $this->objectTypes = array('user', 'ogroup');
    }
    parent::__construct();
  }
}

/*!
 * \brief User, user group and/or role selection
 */
class userGroupRoleSelect extends userGroupSelect
{
  protected $objectTypes = array('user', 'group', 'role', 'ogroup');
}
?>
