  <link rel="stylesheet" type="text/css" href="{filePath file="style.css"}" media="screen"/>
  <link rel="stylesheet" type="text/css" href="{filePath file="form.css"}" media="screen"/>
  <link rel="stylesheet" type="text/css" href="{filePath file="datepicker.css"}" media="screen"/>
  <link rel="stylesheet" type="text/css" href="{filePath file="menu.css"}" media="screen"/>
  <link rel="stylesheet" type="text/css" href="{filePath file="lists.css"}" media="screen"/>
  <link rel="stylesheet" type="text/css" href="{filePath file="tabs.css"}" media="screen"/>
  <link rel="stylesheet" type="text/css" href="{filePath file="plugin.css"}" media="screen"/>
  <link rel="stylesheet" type="text/css" href="{filePath file="setup.css"}" media="screen"/>
  <link rel="stylesheet" type="text/css" href="{filePath file="theme.css"}" media="screen"/>
  <link rel="stylesheet" type="text/css" href="{filePath file="dialog.css"}" media="screen"/>
{foreach from=$css_files item=file}
  <link rel="stylesheet" type="text/css" href="{$file}"/>
{/foreach}
