<div class="setup-header">
  <div id="header-left">
    <img id="fd-logo" class="optional" src="geticon.php?context=applications&amp;icon=fusiondirectory&amp;size=48" alt="FusionDirectory logo"/>
  </div>
  <div id="header-right">
  </div>
</div>

{* FusionDirectory - smarty template for setup header, which is also used by login and password recovery *}
