/********************************
 *           L i s t s          *
 ********************************/
table.listingTable {
border-top: 1px solid #aaa;
border-bottom: 1px solid #aaa;
border-left: 1px solid #aaa;
border-spacing: 0;
}

table.listingTable thead tr {
position: relative;
height: auto;
margin: 3px;
padding: 1px;
white-space: nowrap;
}

table.listingTable > thead > tr > th {
box-sizing: content-box;
background: #f0f0f0;
font-weight: bold;
border-bottom: 1px solid #aaa;
padding: 3px;
height: 22px;
}
html.ltr table.listingTable > thead > tr > th {
border-right: 1px solid #aaa;
}
html.rtl table.listingTable > thead > tr > th {
border-left: 1px solid #aaa;
}

/* Up and Down Arrows */
.listingTable > thead > tr > th.descend:after {
content: "\25B2";
}
.listingTable > thead > tr > th.ascend:after {
content: "\25BC";
}

table.listingTable > tbody {
height: 100%;
overflow-x: hidden;
overflow-y: auto;
}

table.listingTable > tbody > tr {
height: auto;
white-space: nowrap;
}

table.listingTable > tbody > tr:nth-child(odd) {
  background-color: #fff;
}

table.listingTable > tbody > tr:nth-child(even) {
  background-color: #f5f5f5;
}

table.listingTable > tbody > tr.entry-locked:nth-child(odd) {
  background-color: #ffc;
}

table.listingTable > tbody > tr.entry-locked:nth-child(even) {
  background-color: #f5f5cc;
}

html.ltr table.listingTable > tbody > tr td:last-child {
padding-right: 5px;
}
html.rtl table.listingTable > tbody > tr td:last-child {
padding-left: 5px;
}

table.listingTable > tbody > tr:hover {
  background-color: #ddd;
}

table.listingTable > tbody > tr.entry-locked:hover {
  background-color: #dda;
}

table.listingTable > tbody > tr td {
color: #000;
margin: 3px;
padding: 2px;
word-wrap: break-word;
white-space: normal;
max-width: 500px;
background: transparent;
height: 22px;
}
html.ltr table.listingTable > tbody > tr td {
border-right: 1px solid #aaa;
}
html.rtl table.listingTable > tbody > tr td {
border-left: 1px solid #aaa;
}

table.listingTable > tbody > tr td > input[type=image],
table.listingTable > tbody > tr td > img {
padding: 2px;
}


/* List border */

div.nlistFooter {
background-color: #e5e5e5;
border-left: 1px solid silver;
border-right: 1px solid silver;
border-bottom: 1px solid silver;
padding: 3px;
width: auto;
}

/* Tree List (used in baseSelector) */
ul.treeList,ul.treeList ul {
list-style-type: none;
background: url(../../images/lists/vline.png) repeat-y;
margin: 0;
padding: 0;
}

html.ltr ul.treeList ul {
margin-left: 10px;
}
html.rtl ul.treeList ul {
margin-right: 10px;
}

ul.treeList a:hover {
background-color: #ddd;
}

a.treeList {
padding: 2px;
cursor: pointer;
}

a.treeListSelected {
font-weight: bold;
color: #1010af;
background-color: #ddd;
padding: 2px;
cursor: pointer;
}

a.treeList:hover,a.treeListSelected:hover {
background-color: #ddd;
padding: 2px;
}

ul.treeList a {
padding: 2px;
cursor: pointer;
}

ul.treeList li {
margin: 0;
padding: 0 12px;
line-height: 20px;
background: url(../../images/lists/node.png) no-repeat;
}

li.treeListSelected a {
font-weight: bold;
color: #1010af;
padding: 2px;
}

ul.treeList li.last,
ul.treeList li:last-child {
background: #fff url(../../images/lists/lastnode.png) no-repeat;
}

div.treeList {
background-color: #fff;
border: 1px solid #aaa;
padding: 5px;
position: absolute;
z-index: 500;
overflow-y: auto;
float: left;
}
html.rtl div.treeList {
float: right;
}

table.listing-container {
width: 100%;
height: 100%;
vertical-align: top;
text-align: left;
border: none;
border-spacing: 2px;
}
html.rtl table.listing-container {
text-align: right;
}

table.listing-container > tbody > tr > td.list {
height: 100%;
vertical-align: top;
}

table.listing-container > tbody >  tr > td.filter {
max-width: 300px;
vertical-align: top;
}

table.listing-container > tbody >  tr > td.filter .contentboxb {
min-width: 230px;
}

/* On small screens */
@media (max-width: 640px) {
  table.listingTable {
  table-layout: auto;
  }
  table.listingTable > tbody > tr > td, table.listingTable > thead > tr > th {
  width: auto !important;
  min-width: 0 !important;
  }
  html.ltr table.listingTable > tbody > tr td:last-child {
  padding-right: 1px;
  }
  html.rtl table.listingTable > tbody > tr td:last-child {
  padding-left: 1px;
  }
  table.listing-container {
  border: none;
  border-spacing: 0;
  border-collapse: collapse;
  }
  table.listing-container > tbody > tr > td.list, table.listing-container > tbody >  tr > td.filter {
  width: 100%;
  min-width: 0;
  max-width: none;
  height: auto;
  display: block;
  padding: 0;
  }
  table.listing-container > tbody > tr > td.list > div, table.listing-container > tbody >  tr > td.filter > div {
  border: none;
  }
  table.listing-container > tbody > tr > td.list:after, table.listing-container > tbody >  tr > td.filter:after {
  content: " ";
  display: block;
  }
  table.listing-container > tbody >  tr > td.filter input[type=submit] {
  border: 1px solid #ddd;
  width: 100% !important;
  padding: 10px;
  display: inline-block;
  font-weight: bold;
  }
  html.ltr table.listing-container > tbody >  tr > td.filter input[type=submit]:active {
  position: relative;
  top: 1px;
  left: 1px;
  }
  html.rtl table.listing-container > tbody >  tr > td.filter input[type=submit]:active {
  position: relative;
  top: 1px;
  right: 1px;
  }
}
