/* Login screen */
p.fusiondirectoryLoginWarning {
text-align: center;
color: red;
font-weight: bold;
font-size: 12px;
}

html, body {
min-height: 100%;
}

div#window-container {
text-align: center;
box-shadow: 0 0 2px black;
width: 100%;
bottom: 0;
top: 52px;
position: absolute;
}

div#window-div {
min-width: 50%;
margin-top: 100px;
border: 1px solid #aaaaaa;
background-color: white;
display: inline-block;
border-radius: 10px;
box-shadow: 0 0 3px black inset;
}

div#window-titlebar {
text-align: center;
border-bottom: 1px solid #aaaaaa;
padding: 8px;
background-color: #dbe6c6;
border-radius: 10px 10px 0 0;
}

div#window-titlebar > img {
float: left;
height:30px;
}

div#window-titlebar > p {
margin: 0;
padding: 0;
font-size: 24px;
font-weight: bold;
}

div#window-content > div {
padding: 10px;
}

div#window-footer {
padding: 0;
}

div#window-footer > div {
display: inline-block;
width: 47%;
padding: 1%;
}

div#window-footer > div:nth-child(1) {
color: red;
font-weight: bold;
}
html.ltr div#window-footer > div:nth-child(1) {
text-align: left;
}
html.rtl div#window-footer > div:nth-child(1) {
text-align: right;
}

span.warning, p.warning {
color: red;
}

div.copynotice {
position: fixed;
bottom: 0;
right: 5px;
border: none;
}

div.copynotice a {
color: #31363b;
}

/* On small screens */
@media (max-width: 640px) {
  div#window-div {
  min-width: 70%;
  margin-top: 10px;
  }
  div#window-footer > div {
  display: block;
  width: auto;
  padding: 5px;
  text-align: center !important;
  }
  div#window-footer.plugbottom > div:nth-child(2) > input[type=submit] {
  width: 100% !important;
  }
  p.fusiondirectoryLoginWarning {
  margin-top: 0;
  margin-bottom: 0;
  }
}
